# -------------------------------------------------------------------
# set of CMAKE Macros for Polychrony Toolset.
# Author: Loc Besnard 
# Email: Loic.Besnard@irisa.fr
# -------------------------------------------------------------------
# SYNOPSIS
#    cmake <path-to-source> [-DDEBUG_MODE=TRUE]
# -------------------------------------------------------------------
# It includes the CMAKE files: LIB.cmake, LIBJAVA.cmake, LIBTex.cmake
# -------------------------------------------------------------------

SET(SIGNAL_TOOLBOX_VERSION "4.23")
SET(SIGNAL_GUI_VERSION "4.1")
SET(SOFTNAME PolychronyToolset)
SET(pkEnvFile ${SOFTNAME}_setup)
SET(PREDEF_PKCONFIG_NAME _config_)
SET(PREDEF_PKCONFIG_REC_NAME _configRec_)
SET(PREDEF_PKINSTALL_NAME make${SOFTNAME})
SET(PREDEF_PKINSTALL_CMD_FILE _cmd_${PREDEF_PKINSTALL_NAME})
SET(PREDEF_DEFS_FILENAME_CMAKE defs.cmake)
SET(PREDEF_PKEXPORT_NAME export${SOFTNAME})
SET(CONFIGURE_FILE_NAME configure)

SET(GLOB_PK_MAKE_OPTIONS_DOC doc)
SET(GLOB_PK_MAKE_OPTIONS force source clean all )

SET(LOCALNAME_CROSSREF_DATA_BASE \${pK_tmp}/CrossRefs/_DB_/pK_ALL_OBJECTS_CrossRefs.dot)

IF(UNIX)
  SET(OS_EXEC_SCRIPT "source")
ELSE(UNIX)
  SET(OS_EXEC_SCRIPT "call")
ENDIF(UNIX)

#SET(EXPORT_FUNC_CMAKE "${pK_ROOT}/CMakeModules/FuncExport.cmake")

SET(pKMacros_FOUND TRUE)
SET(CMAKE_VERBOSE_MAKEFILE ON)
SET_PROPERTY(GLOBAL PROPERTY ALLOW_DUPLICATE_CUSTOM_TARGETS 1)

INCLUDE(${CMAKE_MODULE_PATH}/LIB_IO.cmake)
INCLUDE(${CMAKE_MODULE_PATH}/LIB_COMPIL.cmake)
INCLUDE(${CMAKE_MODULE_PATH}/LIB.cmake)
INCLUDE(${CMAKE_MODULE_PATH}/LIBJAVA.cmake)
INCLUDE(${CMAKE_MODULE_PATH}/LIBTex.cmake)


# This variable is used to rename the exported packages.
GET_SYSTEM_INFOS(VSYSTEM_NAME VSYSTEM_VERSION)
SET(G_EXPORTED_PACKAGE_NAME ${SOFTNAME}-SignalToolBox-${SIGNAL_TOOLBOX_VERSION}-${VSYSTEM_NAME}-${VSYSTEM_VERSION})
SET(G_EXPORTED_PACKAGE_NAME_GUI ${SOFTNAME}-SignalGUI-${SIGNAL_TOOLBOX_VERSION}-${VSYSTEM_NAME}-${VSYSTEM_VERSION})

# -----------------------------------------------------------
# Defining:
#  the ROOT path (pK_ROOT) of the Polychrony toolset software
#  the NATIVE OS ROOT path (NATIVE_pK_ROOT)
# -----------------------------------------------------------
MACRO(SETXKROOT)
  FILE(TO_CMAKE_PATH "$ENV{pK_ROOT}" pK_ROOT)
  IF(UNIX)
    SET(NATIVE_pK_ROOT ${pK_ROOT})
  ELSE(UNIX) #WIN32
    FILE(TO_NATIVE_PATH ${pK_ROOT} NATIVE_pK_ROOT_1)
    STRING(REPLACE "\\" "\\\\" NATIVE_pK_ROOT ${NATIVE_pK_ROOT_1})
  ENDIF(UNIX)
ENDMACRO(SETXKROOT)

# -----------------------------------------------------------
# Defining the variable VAR_SETUP_PK that defines
#  the command to call for the Polychrony setup such that
#   - source pK_ROOT/PolychronyToolset_setup  (Linux, tcsh)
#   - call pK_ROOT\PolychronyToolset_setup.bat  (windoxs)
# -----------------------------------------------------------
MACRO(MK_CMD_TO_CALL_FOR_SETUP)
  SET(V_X "${pK_ROOT}/${pkEnvFile}")
  IF(UNIX)
    SET(VAR_SETUP_PK "source ${V_X}")
  ELSE(UNIX) #WIN32
    FILE(TO_NATIVE_PATH ${V_X} NATIVE_V_X_1)
    STRING(REPLACE "\\" "\\\\" NATIVE_V_X ${NATIVE_V_X_1})
    SET(VAR_SETUP_PK  "call ${NATIVE_V_X}")
  ENDIF(UNIX)
ENDMACRO(MK_CMD_TO_CALL_FOR_SETUP)


# -------------------------------------------------------------------------------------------------------
# Is adds some properties to a TARGET and it declares it for an export (in the file ${EXPORT_FUNC_CMAKE}).
# It assumes that the variable LIBRARY_OUTPUT_PATH is set to the address of the library.
# -------------------------------------------------------------------------------------------------------
MACRO(PK_ADD_STD_PROPERTIES TARGET)
#  MESSAGE( " TARGET = ${TARGET}")
IF (NOT MSVC)
  SET_TARGET_PROPERTIES(${TARGET} PROPERTIES COMPILE_FLAGS ${CPP_FLAGS})
ENDIF (NOT MSVC)
  EXPORT(TARGETS ${TARGET} FILE ${LIBRARY_OUTPUT_PATH}/${TARGET}_export.cmake)
ENDMACRO(PK_ADD_STD_PROPERTIES TARGET)


# -------------------------------------------------------------------------------------------------
# It defines CMAKE variables (pK_ROOT: root of the installation) and general CMAKE parameters.
# -------------------------------------------------------------------------------------------------
MACRO(COMMON_LANG_ENV)
  SETXKROOT()
  MK_CMD_TO_CALL_FOR_SETUP()
  LOAD_USER_CMAKE_DEFS(${pK_ROOT})
  SET_CMAKE_PARAMETERS()
ENDMACRO(COMMON_LANG_ENV)


# ---------------------------------------------------------------------------------------
# It defines CMAKE variables 
#     - for using C/C++ (compiler, linker, defines, ...)
#     - for using/generating command files
#  Defined symbols (used for compiling -D options) LITTLEENDIAN
# ---------------------------------------------------------------------------------------
MACRO(LOAD_ENV)
  PRINT("C_COMPILER = ${CMAKE_C_COMPILER}")
  PRINT("CPP_COMPILER = ${CMAKE_CXX_COMPILER}")
  PRINT("CMAKE_MODULE_PATH = ${CMAKE_MODULE_PATH}")

  INIT_CONFIG()
  SET_SHELL_DEFS()
  COMMON_LANG_ENV()
  SET_C_CPP_DEFS()
  SET_HOST_DEFS()
  SETLITTLEENDIAN()
  SET_CPP_SYMBOLS()
ENDMACRO(LOAD_ENV)

# ---------------------------------------------------------------------------------------
# It defines CMAKE variables for using JAVA and requires JAVA environment.
# ---------------------------------------------------------------------------------------
MACRO(LOAD_ENV_JAVA)
  INIT_CONFIG()
  COMMON_LANG_ENV()
  FIND_PACKAGE(Java REQUIRED)
ENDMACRO(LOAD_ENV_JAVA)

# ---------------------------------------------------------------------------------------
# It generates the setup file for the Polychrony Toolset
# The execution of this file will define the shell variables of all the sofware packages.
# ---------------------------------------------------------------------------------------
MACRO(GEN_BOOT_SIGNAL_TOOLBOX)
  MK_CONFIG_NAME(${pkEnvFile} NAME_FILE_CONFIG)
  IF (PKCONFIG)
    FILE(REMOVE ${NAME_FILE_CONFIG})
    TEX_INIT_SIGNALTOOLBOX_DOCREGISTER()
  ENDIF (PKCONFIG)

  IF(NOT EXISTS ${NAME_FILE_CONFIG})
    BEGIN_CONFIG_FILE(${NAME_FILE_CONFIG})
    MK_SYSTEM_PATH("$ENV{pK_ROOT}" XX)
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} pK_ROOT "${XX}" "")
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} pK_VERSION ${SIGNAL_TOOLBOX_VERSION} "")
    ADD_LOAD_SETENV(${NAME_FILE_CONFIG} pK_ROOT)
  ENDIF(NOT EXISTS ${NAME_FILE_CONFIG})
ENDMACRO(GEN_BOOT_SIGNAL_TOOLBOX)

# -------------------------------------------------------------------------------------------------------
# FOR UNIX ONLY.
# It generates in FILENAME the script defining the producing of the compiling units using the PKMAKE make. 
#  MKF_FILE: file giving the description of the building (mkf file) of the compiling units.
#  LOG_FILE: log file of the pkmake analysis
#  COM_FILE: name of the command to execute to produce the compiling units.
#  LOG_COM_FILE: log file of the execution of the COM_FILE
#  ------------------------------------------------------------------------------------------------------
MACRO(pK_GENCOMMAND_MKF FILENAME PK_MAKE MKF_FILE LOG_FILE COM_FILE LOG_COM_FILE)
  IF(UNIX)
    GEN_HEADER_SCRIPT_SILENT(${FILENAME})
    FILE(APPEND ${FILENAME} "\\rm -f ${LOG_FILE}\n")
    FILE(APPEND ${FILENAME} "${PK_MAKE} < ${MKF_FILE} > ${LOG_FILE}\n")
    FILE(APPEND ${FILENAME} "if ($status == 0) then\n")
    FILE(APPEND ${FILENAME} "  chmod  u+rx ${COM_FILE}\n")
    FILE(APPEND ${FILENAME} "  \\rm -f ${LOG_COM_FILE}\n")
    FILE(APPEND ${FILENAME} "  ${COM_FILE} > ${LOG_COM_FILE}\n")
    FILE(APPEND ${FILENAME} "else\n")
    FILE(APPEND ${FILENAME} "  echo \"ERROR   +++---> ${PK_MAKE} < ${MKF_FILE} > ${LOG_FILE} failed\"\n")
    FILE(APPEND ${FILENAME} "  echo \"check stdin= ${MKF_FILE}\"\n")
    FILE(APPEND ${FILENAME} "  grep  \"MKF\ ERROR\" ${LOG_FILE}\n")
    FILE(APPEND ${FILENAME} "endif\n")
  ENDIF(UNIX)
ENDMACRO(pK_GENCOMMAND_MKF FILENAME PK_MAKE MKF_FILE LOG_FILE COM_FILE LOG_COM_FILE)

# -----------------------------------------------------------------------------------
# FOR UNIX ONLY. Producing the compiling units using PKMAKE
# example : MK_SOURCE(${Signal_package} makeSignalSrc)
# will generate the code to rebuild the source defined in the (pkMake) file : ${DIR_MKF}/${PREFIX_MKF_FILE}.mkf 
# It assumes that the code to produce the source is generated in ${pK_tmp}/${PREFIX_MKF_FILE}.com (this name exists in ${DIR_MKF}/${PREFIX_MKF_FILE}.mkf)
# The log file are generated in the ${pK_tmp}/${PREFIX_MKF_FILE}.log, and ${pK_tmp}/${PREFIX_MKF_FILE}Com.log.
# -----------------------------------------------------------------------------------
MACRO(MK_SOURCE DIR_MKF PREFIX_MKF_FILE)
  if(UNIX)
    SET(PK_MAKE ${pKMake_bin}/pKMAKE)
    SET(MKF_FILE ${DIR_MKF}/${PREFIX_MKF_FILE}.mkf)
    SET(LOG_FILE ${pK_tmp}/${PREFIX_MKF_FILE}.log)
    SET(COM_FILE ${pK_tmp}/${PREFIX_MKF_FILE}.com)
    SET(LOG_COM_FILE ${pK_tmp}/${PREFIX_MKF_FILE}Com.log)
    SET(FILENAME ${CMAKE_CURRENT_BINARY_DIR}/${PREFIX_MKF_FILE}_cmd_.sh)
    pK_GENCOMMAND_MKF(${FILENAME} ${PK_MAKE} ${MKF_FILE} ${LOG_FILE} ${COM_FILE} ${LOG_COM_FILE})
    ADD_CUSTOM_TARGET(source
      COMMAND ${CMAKE_COMMAND} -E echo_append "Building the source using pKMake from ${MKF_FILE}"
      COMMAND ${CMAKE_COMMAND} -E echo " "
      WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
      COMMAND chmod  u+rx ${FILENAME}
      COMMAND "${FILENAME}"
#      COMMAND ${CMAKE_COMMAND} -E echo " "
      )
  ENDIF(UNIX)
ENDMACRO(MK_SOURCE MKF_FILE)

# Define the src_clean target (the files of the directory DIR are deleted).
MACRO(MK_TARGET_CLEAN_SRC DIR)
  SET(TO_DELETE "${DIR}/*")
  IF(UNIX)
    ADD_CUSTOM_TARGET(src_clean
      COMMAND ${CMAKE_COMMAND} -E echo_append "Removing the files ${TO_DELETE}"
      COMMAND ${CMAKE_COMMAND} -E echo " "
      COMMAND ${CMAKE_COMMAND} -E remove -f "${TO_DELETE}"
      COMMAND ${CMAKE_COMMAND} -E echo "done"
     )
   ELSE(UNIX)
     ADD_CUSTOM_TARGET(src_clean
	COMMAND ${CMAKE_COMMAND} -E echo_append "No actions fo src_clean target: the files ${TO_DELETE} will not be deleted."
	COMMAND ${CMAKE_COMMAND} -E echo " "
	)
  ENDIF(UNIX)
ENDMACRO(MK_TARGET_CLEAN_SRC DIR)

# Define the src_clean target (the files $FILES are deleted).
MACRO(MK_TARGET_CLEAN_SRC_2DIRS DIR1 DIR2)
  IF(UNIX)
    ADD_CUSTOM_TARGET(src_clean
      COMMAND ${CMAKE_COMMAND} -E echo_append "Removing the files of ${DIR1},  ${DIR2} "
      COMMAND ${CMAKE_COMMAND} -E echo " "
      COMMAND ${CMAKE_COMMAND} -E remove -f "${DIR1}/*" "${DIR2}/*"
      COMMAND ${CMAKE_COMMAND} -E echo "done"
     )
   ELSE(UNIX)
     ADD_CUSTOM_TARGET(src_clean
	COMMAND ${CMAKE_COMMAND} -E echo_append "No actions fo src_clean target: the files of ${DIR1}, ${DIR2} will not be deleted."
	COMMAND ${CMAKE_COMMAND} -E echo " "
	)
  ENDIF(UNIX)
ENDMACRO(MK_TARGET_CLEAN_SRC_2DIRS DIR1 DIR2)



MACRO(MK_EMPTY_SOURCE_TARGET VPROJECT)
  if(UNIX)
    ADD_CUSTOM_TARGET(source
      COMMAND ${CMAKE_COMMAND} -E echo_append "Project ${VPROJECT}: the compiling units are assumed to be up to date."
      COMMAND ${CMAKE_COMMAND} -E echo " "
      )
    # no actions for the internal target src_clean
     ADD_CUSTOM_TARGET(src_clean
	COMMAND ${CMAKE_COMMAND} -E echo " "
	)
  ENDIF(UNIX)
ENDMACRO(MK_EMPTY_SOURCE_TARGET VPROJECT)



MACRO(MK_EMPTY_SOURCE_TARGET VPROJECT)
  if(UNIX)
    ADD_CUSTOM_TARGET(source
      COMMAND ${CMAKE_COMMAND} -E echo_append "Project ${VPROJECT}: the compiling units are assumed to be up to date."
      COMMAND ${CMAKE_COMMAND} -E echo " "
      )
    # no actions for the internal target src_clean
     ADD_CUSTOM_TARGET(src_clean
	COMMAND ${CMAKE_COMMAND} -E echo " "
	)
  ENDIF(UNIX)
ENDMACRO(MK_EMPTY_SOURCE_TARGET VPROJECT)


#  -----------------------------------------------------------------------------------
# It generates (OS dependent) the command used to load a set of shell variable in the 
# user environment (source for tcsh, call for windows)
#  -----------------------------------------------------------------------------------
MACRO(GEN_LOAD_ENVIRONMENT CONFIG_FILE)
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} " if ( ! \$?pK_ROOT ) then \n")
    FILE(APPEND ${CONFIG_FILE} "   echo \"  loading environment\"\n")
    FILE(APPEND ${CONFIG_FILE} "   ${OS_EXEC_SCRIPT} ${pK_ROOT}/${pkEnvFile} > /dev/null\n")
    FILE(APPEND ${CONFIG_FILE} " endif\n")
    FILE(APPEND ${CONFIG_FILE} "\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "${OS_EXEC_SCRIPT} ${pkEnvFile} \n")
  ENDIF(UNIX)
ENDMACRO( GEN_LOAD_ENVIRONMENT CONFIG_FILE)

# -----------------------------------------------------------------------------------
# It returns in CONFIG_FILE the address of the name of the configuration 
# file (given by PREDEF_PKCONFIG_NAME - a global variable-) in the directory DIR. 
# The name is OS dependent (for example, it is suffixed by .bat for Windows).
# -----------------------------------------------------------------------------------
MACRO(GET_STD_CONFIG_NAME DIR CONFIG_FILE)
  MK_CONFIG_NAME(${PREDEF_PKCONFIG_NAME} CONFIG_NAME)
  SET(CONFIG_FILE ${DIR}/${CONFIG_NAME})
ENDMACRO (GET_STD_CONFIG_NAME DIR CONFIG_FILE)

# -----------------------------------------------------------------------------------
# It returns in CONFIG_FILE the address of the name of the configuration 
# file (given by PREDEF_PKCONFIG_NAME - a global variable-) in the directory DIR. 
# The name is OS dependent (for example, it is suffixed by .bat for Windows).
# -----------------------------------------------------------------------------------
MACRO(GET_CONFIGREC_NAME DIR CONFIG_FILEREC)
  #  MK_CONFIG_NAME(${PK_CONFIG_FILE_REC} CONFIG_NAME)
  #  SET(${CONFIG_FILEREC} ${DIR}/${CONFIG_NAME})
  SET(${CONFIG_FILEREC} ${DIR}/${PK_CONFIG_FILE_REC})
ENDMACRO (GET_CONFIGREC_NAME DIR CONFIG_FILEREC)

# -------------------------------------------------------------------------------------------------------
# It returns 
#    - in CMD the os (dependent) name of a command NAME (suffixed by .bat for Windows for example)
#    - in OPTIONS the standard options available for the make command for the Polychrony toolset.
#    - in  DEFAULT_OPTIONS the default options available for the make command for the Polychrony toolset.
# -------------------------------------------------------------------------------------------------------
MACRO(GET_STD_BUILDCOMMAND NAME CMD OPTIONS DEFAULT_OPTIONS)
  MK_CONFIG_NAME(${NAME} CMD)
  SET(OPTIONS ${GLOB_PK_MAKE_OPTIONS} ${GLOB_PK_MAKE_OPTIONS_DOC})
  SET(DEFAULT_OPTIONS "all")
ENDMACRO(GET_STD_BUILDCOMMAND NAME CMD OPTIONS DEFAULT_OPTIONS)

# --------------------------------------------------------------------------------
# It generates the ${CMD_NAME} command in the pK_ROOT/com directory.
# The command is useful for building from scratch  the polychronyToolSet
# --------------------------------------------------------------------------------
MACRO(GEN_CLEAN_CMD CMD_NAME SELECTEDDIR FILETOREMOVE)
  MK_CONFIG_NAME(${CMD_NAME} CONFIG_NAME)
  SET(CONFIG_FILE ${pK_com}/${CONFIG_NAME})
  GEN_HEADER_SCRIPT(${CONFIG_FILE})
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "set HERE=`pwd`\n")
    FILE(APPEND ${CONFIG_FILE} "set DIRLIST=`find \$HERE -name ${SELECTEDDIR} -type d -print `\n")
    FILE(APPEND ${CONFIG_FILE} "foreach aDir ( \$DIRLIST )\n")
    FILE(APPEND ${CONFIG_FILE} "  cd \$aDir\n")
    FILE(APPEND ${CONFIG_FILE} "  chmod -f -R +w *\n")
    FILE(APPEND ${CONFIG_FILE} "  \\rm  -f -R ${FILETOREMOVE} \n")
    FILE(APPEND ${CONFIG_FILE} "  cd \$HERE\n")
    FILE(APPEND ${CONFIG_FILE} "end\n")
  ELSE(UNIX)
    # Not implemented.
  ENDIF(UNIX)
ENDMACRO(GEN_CLEAN_CMD)

# --------------------------------------------------------------------------------
# It generates the clean_cmake, clean_doc commands in the pK_ROOT/com directory.
# The command is useful for managing the polychrony ToolSet.
# --------------------------------------------------------------------------------
MACRO(GEN_CLEAN_COMMANDS)
  GEN_CLEAN_CMD(pk_clean_cmake cmake *)
  GEN_CLEAN_CMD(pk_clean_doc html index.html)
ENDMACRO(GEN_CLEAN_COMMANDS)

# --------------------------------------------------------------------------------
# It generates general infos to the user (how to build/use the environment)
# --------------------------------------------------------------------------------
MACRO(GEN_MESSAGE_INSTALL)
  IF(UNIX)
    SET(XY ./${PREDEF_PKINSTALL_NAME})
  ELSE(UNIX)
    SET(XY ${PREDEF_PKINSTALL_NAME}.bat)
  ENDIF(UNIX)

  MESSAGE_HEADER_LINE()
  MESSAGE_PARAGRAPH_LINE("Make files have been built. To complete the PolychronyToolset installation")
  MESSAGE_SUB_LINE("  return to the directory root and execute the completion command:")
  IF(UNIX)
    MESSAGE_COMMAND_LINE("cd ${NATIVE_pK_ROOT}\;${XY}")
  ELSE(UNIX)
    MESSAGE_COMMAND_LINE("cd ${NATIVE_pK_ROOT} & ${XY}")
#   MESSAGE_COMMAND_LINE("${XY}")
  ENDIF(UNIX)
  MESSAGE_PARAGRAPH_LINE("Then to make PolychronyToolset source documentation available execute: ")
  MESSAGE_COMMAND_LINE("${XY} doc")
  MESSAGE_PARAGRAPH_LINE("To get help for using ${XY} ")
  MESSAGE_COMMAND_LINE("${XY} -h")
  GEN_MESSAGE_END()
ENDMACRO(GEN_MESSAGE_INSTALL)

# --------------------------------------------------------------------------------------------------------------------
# It adds the alias command ${CMD} to ${CMD_FILE} in the configuration file CONFIG_FILE
# It also adds the message that explains how to use this command. The message is parametrized by the OPTIONS and TEXT
#
# Example: with CMD= makeTrees2Kernel, 
#               CMD_FILE= /home/lbesnard/Polychrony/V4.17/Linux/Signal/package/Trees2Kernel/makeTrees2Kernel
#               OPTIONS= source clean all doc, 
#               TEXT=Trees2Kernel package, 
# the following lines are generated (in the file denoted by CONFIG_FILE)
#    echo ">>>> use makeTrees2Kernel [ source clean all doc] to build a new Trees2Kernel package"
#    alias makeTrees2Kernel /home/lbesnard/Polychrony/V4.17/Linux/Signal/package/Trees2Kernel/makeTrees2Kernel $*
# --------------------------------------------------------------------------------------------------------------------
MACRO(DEFINE_CMD_IN_CONFIG_FILE CONFIG_FILE CMD OPTIONS CMD_FILE TEXT)
  IF(UNIX)
    LISTOPTIONS2STRING("${OPTIONS}" STROPTIONS)
    GEN_ECHO_USE(${CONFIG_FILE} "${CMD}" "[${STROPTIONS}]" "to build a new ${TEXT}" )
    FILE(APPEND ${CONFIG_FILE} "alias ${CMD} ${CMD_FILE} \$*\n ")
  ENDIF(UNIX)
ENDMACRO(DEFINE_CMD_IN_CONFIG_FILE CONFIG_FILE CMD OPTIONS CMD_FILE TEXT)

FUNCTION(GEN_GPACK_COMMAND CONFIG_FILE GEN)
  IF(UNIX) 
    SET(LOCGEN "TGZ")
  ELSE(UNIX) 
    SET(LOCGEN "TGZ")
    # SET(LOCGEN "NSIS") 
  ENDIF(UNIX) 
  FILE(APPEND ${CONFIG_FILE} "cpack -G ${LOCGEN} \n")
  SET(${GEN} ${LOCGEN} PARENT_SCOPE)
  
ENDFUNCTION( GEN_GPACK_COMMAND CONFIG_FILE GEN)

# Generating the file ${PREDEF_PKEXPORT_NAME}[.bat]
MACRO( GEN_EXPORT_SCRIPT )
  MK_CONFIG_NAME(${PREDEF_PKEXPORT_NAME} CONFIG_NAME)
  SET(CONFIG_FILE ${pK_ROOT}/${CONFIG_NAME})
  GEN_HEADER_SCRIPT(${CONFIG_FILE})
  GEN_REQUIRE_DIR(${CONFIG_FILE} pK_Signal)
  GEN_REQUIRE_DIR(${CONFIG_FILE} pK_SignalGUI)
  GEN_REQUIRE_DIR(${CONFIG_FILE} pK_InteEclipse)
  GEN_REQUIRE_DIR(${CONFIG_FILE} pK_export)

  GEN_DEF_HERE(${CONFIG_FILE})
  # Cleaning the export directory.
  GEN_DELETE(${CONFIG_FILE} ${pK_export})
  
  # Bach Compiler.
  GEN_GOTO_DIR_DIR(${CONFIG_FILE} pK_Signal cmake)
  GEN_GPACK_COMMAND(${CONFIG_FILE} MYCPACKGENERATOR)
  GEN_MOVE_RES(${CONFIG_FILE} "pK_Signal"  "${G_EXPORTED_PACKAGE_NAME}" ${MYCPACKGENERATOR})

  GEN_EXPORT_FILE(pK_doc Polychrony_SignalToolBox_install.pdf)

  # GUI
  GEN_GOTO_DIR_DIR(${CONFIG_FILE} pK_SignalGUI cmake)
  GEN_GPACK_COMMAND(${CONFIG_FILE} MYCPACKGENERATOR)
  GEN_MOVE_RES(${CONFIG_FILE} "pK_SignalGUI" "${G_EXPORTED_PACKAGE_NAME_GUI}" ${MYCPACKGENERATOR})
  GEN_CLEAN_EXPORTDIR(${CONFIG_FILE} "pK_SignalGUI")
  # For POP. POP IS INTEGRATED WITH THE SIGNALTOOLBOX. (June 2015, LBesnard)
  # GEN_GOTO_DIR_DIR(${CONFIG_FILE} pK_InteEclipse cmake)
  # GEN_GPACK_COMMAND(${CONFIG_FILE} MYCPACKGENERATOR)
  # GEN_MOVE_RES(${CONFIG_FILE} "POP" "pK_InteEclipse" "Signal" ${MYCPACKGENERATOR})
  # GEN_CLEAN_EXPORTDIR(${CONFIG_FILE} "pK_InteEclipse")

  GEN_CLEAN_EXPORTDIR(${CONFIG_FILE} "pK_Signal")
  GEN_CHDIR_HERE(${CONFIG_FILE})

ENDMACRO( GEN_EXPORT_SCRIPT )

# Generates the ${configureFileName}[.bat] file.
FUNCTION(GEN_MAKEFILES_SCRIPT)
  MK_CONFIG_NAME(${CONFIGURE_FILE_NAME} CONFIG_NAME)
  SET(CONFIG_FILE ${pK_ROOT}/${CONFIG_NAME})
  GEN_HEADER_SCRIPT(${CONFIG_FILE})
  FILE(APPEND ${CONFIG_FILE} "cd cmake\n")
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "unsetenv pK_ROOT\n")
    FILE(APPEND ${CONFIG_FILE} "cmake ..\n")
    FILE(APPEND ${CONFIG_FILE} "set result=$status\n")
    FILE(APPEND ${CONFIG_FILE} "cd ..\n")
    FILE(APPEND ${CONFIG_FILE} "exit $result\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "set pK_ROOT=\n")
    FILE(APPEND ${CONFIG_FILE} "cmake .. -G \"NMake Makefiles\"\n")
    FILE(APPEND ${CONFIG_FILE} "set result=%ERRORLEVEL%\n")
    FILE(APPEND ${CONFIG_FILE} "cd ..\n")
    FILE(APPEND ${CONFIG_FILE} "exit /B %result%\n")
  ENDIF(UNIX)
ENDFUNCTION(GEN_MAKEFILES_SCRIPT)


INCLUDE(${CMAKE_MODULE_PATH}/LIB_TESTS.cmake)

# -----------------------------------------------------------------------------------
# It generates the ${PREDEF_PKINSTALL_NAME}[.bat] command file. 
# This script must be executed to build the Polychrony ToolSet from scratch.
# -----------------------------------------------------------------------------------
MACRO(GEN_INSTALL_SCRIPT)
  IF(PKCONFIG)
    # initializing the _cmd_makePolychronyTollset file (all the available commands will be generated in this file)
    MK_CONFIG_NAME(${PREDEF_PKINSTALL_CMD_FILE} CONFIG_NAME)
    SET(_NAME_FILE_COMMANDS ${pK_ROOT}/${CONFIG_NAME})
    FILE(REMOVE ${_NAME_FILE_COMMANDS})
    # -----
    GEN_CMD_ECHO (${pK_com} echoInf INFO 1)
    GEN_CMD_ECHO (${pK_com} echoErr ERROR 0)
    GEN_CMD_ECHO (${pK_com} echoWar WARNING 0)
    # Only when the source is exported.
    IF(NOT DEFINED BIN_INSTALL)
      MK_CONFIG_NAME(${PREDEF_PKINSTALL_NAME} CONFIG_NAME)
      SET(CONFIG_FILE ${pK_ROOT}/${CONFIG_NAME})

      SET(CURRENT_DEFS_CMAKE ${pK_ROOT}/${PREDEF_DEFS_FILENAME_CMAKE})
      GEN_HEADER_SCRIPT(${CONFIG_FILE})
      SET(CURRENT_DEFS_CMAKE)

      GEN_HELP_MAKE(${CONFIG_FILE})
      GEN_LIST_MAKE(${CONFIG_FILE})
      GEN_LOAD_ENVIRONMENT(${CONFIG_FILE})

      GEN_SCRIPT_TEST_ARG(${CONFIG_FILE} "doc")
      # The doc target. The makeSignalToolBox must be applied first.
      GEN_CMD_FIRSTCALL_DOCBUILDER(${CONFIG_FILE} "${LOCALNAME_CROSSREF_DATA_BASE}" "\${pK_Signal}/cmake/CMakeCache.txt"  "\$SignalDoc_html/SignalToolBox.tag")
      GEN_INSTALL_CMD(${CONFIG_FILE} pK_Signal makeSignalToolbox)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_Host makeHost)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_Tree makeTree)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_BDD makeBDD)
      GEN_INSTALL_CMD(${CONFIG_FILE} pK_SignalGUI makeSignalGUI)
      GEN_CMD_FIRSTCALL_DOCBUILDER_END(${CONFIG_FILE})
      GEN_SCRIPT_EXIT(${CONFIG_FILE})
      GEN_SCRIPT_END_TEST_ARG(${CONFIG_FILE} "doc")
      GEN_COMMENT(${CONFIG_FILE} "--------------")

      # Building the other targets.
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_Host makeHost)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_pKMake makepKmake)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_CString makeCString)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_PKString makePKString)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_BufTools makeBufTools)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_LogPrinter makeLogPrinter)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_ReaderTools makeReaderTools)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_new_pKMake makenew_pKMake)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_pKPrototypeExtract makepKPrototypeExtract)

      GEN_INSTALL_CMD(${CONFIG_FILE} utl_Tree makeTree)
      GEN_INSTALL_CMD(${CONFIG_FILE} utl_BDD makeBDD)
      GEN_INSTALL_CMD(${CONFIG_FILE} pK_Signal makeSignalToolbox)
      GEN_INSTALL_CMD(${CONFIG_FILE} SignalLib_C makeCexport)
      GEN_INSTALL_CMD(${CONFIG_FILE} SignalLib_Java makeJavaexport)
      GEN_INSTALL_CMD(${CONFIG_FILE} pK_InteEclipse makeInteEclipse)
      GEN_INSTALL_CMD(${CONFIG_FILE} pK_SignalGUI makeSignalGUI)
      GEN_INSTALL_CMD(${CONFIG_FILE} Signal_bin makeSignalData)

# GEN_INSTALL_CMD(${CONFIG_FILE} pK_Examples makeExamples)  --> generated by GEN_CONFIG_EXAMPLES()
      GEN_ECHO (${CONFIG_FILE} "")
      GEN_ECHO (${CONFIG_FILE} "\"##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##---\"")
      GEN_ECHO (${CONFIG_FILE} "\"=> To make system variables available, before using PolychronyToolset execute:\"")
      GEN_ECHO (${CONFIG_FILE} "\"${VAR_SETUP_PK}\"")
      GEN_ECHO (${CONFIG_FILE} "\"# ------------------# ------------------# ------------------# ---------------- #\"")
      #  EXECUTE_PROCESS( COMMAND ${CONFIG_FILE} WORKING_DIRECTORY ${pK_ROOT} )
      GEN_CLEAN_COMMANDS()
      GEN_TESTS_COMMANDS()
      TEX_GEN_DOCREGISTER()

      # ADDING THE COMMAND TO THE ${PREDEF_PKCONFIG_NAME} FILE.
      GET_STD_CONFIG_NAME(${pK_ROOT} CONFIG_FILE)
      GET_STD_BUILDCOMMAND(${PREDEF_PKINSTALL_NAME} CMD OPTIONS DEFAULT_OPTIONS)
      SET(CMD_FILE ${pK_ROOT}/${CMD})
      DEFINE_CMD_IN_CONFIG_FILE(${CONFIG_FILE} ${CMD} "${OPTIONS}" ${CMD_FILE} "${SOFTNAME}")
      GEN_EXPORT_SCRIPT()
      GEN_MAKEFILES_SCRIPT()
    ENDIF(NOT DEFINED BIN_INSTALL)
  ENDIF(PKCONFIG)
ENDMACRO (GEN_INSTALL_SCRIPT)


# --------------------------------------------------------------------------------------------
# In configuration mode (the global variable PKCONFIG is true):
# It generates the file makeFOO at the adress NAME_ADDR_FOO and the documentation...
# It also adds the alias of the command to the configuration file at the adress NAME_ADDR_FOO
#
# Assumed: the directory of the FOO package is a standard one (structure src/export/mod/doc)
# --------------------------------------------------------------------------------------------
MACRO(GEN_CONFIG_STD_PACKAGE NAME_ADDR_FOO FOO PREFIX FOODOCHTML)
IF(PKCONFIG)
  SET(ADRFOO ${${NAME_ADDR_FOO}})
  GET_STD_CONFIG_NAME(${ADRFOO} CONFIG_FILE)
  GET_STD_BUILDCOMMAND(make${FOO} CMD OPTIONS DEFAULT_OPTIONS)
    
  # Generating the command file: makeFoo
  SET(VARS_REQUIRED pK_ROOT pK_com)
  SET(DIRS_VAR_REQUIRED ${NAME_ADDR_FOO} ${PREFIX}obj ${PREFIX}src ${PREFIX}mod)
  GEN_CMD_FILE(${NAME_ADDR_FOO} "${VARS_REQUIRED}" "${DIRS_VAR_REQUIRED}" "${OPTIONS}" "${DEFAULT_OPTIONS}" ${CMD} CMD_FILE)
  # Adding the alias to the configuration file.
  DEFINE_CMD_IN_CONFIG_FILE(${CONFIG_FILE} ${CMD} "${OPTIONS}" ${CMD_FILE} "${FOO} package")
  TEX_NEWCOMMAND_INDEX(${FOO} ${FOODOCHTML} "${FOO} interface")
ENDIF(PKCONFIG)
ENDMACRO(GEN_CONFIG_STD_PACKAGE NAME_ADDR_FOO FOO PREFIX  FOODOCHTML)


# --------------------------------------------------------------------------------------------
# In configuration mode (the global variable PKCONFIG is true):
# It generates the file makeFOO at the adress NAME_ADDR_FOO and the documentation...
# It also adds the alias of the command to the configuration file at the adress NAME_ADDR_FOO
#
# Assumed: the directory of the FOO package is a standard one (structure src/export/mod/doc)
# --------------------------------------------------------------------------------------------
MACRO(GEN_CONFIG_EXAMPLES NAME_ADDR_FOO FOO PREFIX FOODOCHTML)
IF(PKCONFIG)
  SET(ADRFOO ${${NAME_ADDR_FOO}})
  GET_STD_CONFIG_NAME(${ADRFOO} CONFIG_FILE)
  #  GET_STD_BUILDCOMMAND(make${FOO} CMD OPTIONS DEFAULT_OPTIONS) replaced by
  MK_CONFIG_NAME(make${FOO} CMD)
  SET(DEFAULT_OPTIONS test)
  SET(OPTIONS force test)
  
  SET(VARS_REQUIRED pK_ROOT pK_com)
  SET(DIRS_VAR_REQUIRED ${NAME_ADDR_FOO})
  GEN_CMD_FILE_LOC(${NAME_ADDR_FOO} "${VARS_REQUIRED}" "${DIRS_VAR_REQUIRED}" "${OPTIONS}" "${DEFAULT_OPTIONS}" ${CMD} CMD_FILE "EXAMPLE")

  # Adding the alias to the configuration file.
  DEFINE_CMD_IN_CONFIG_FILE(${CONFIG_FILE} ${CMD} "${OPTIONS}" ${CMD_FILE} "${FOO} package")
  TEX_NEWCOMMAND_INDEX(${FOO} ${FOODOCHTML} "${FOO} interface")
ENDIF(PKCONFIG)
ENDMACRO(GEN_CONFIG_EXAMPLES NAME_ADDR_FOO FOO PREFIX  FOODOCHTML)

MACRO(GEN_SIGNALGUI_VERSION NAME_ADDR_FOO)
  IF(PKCONFIG)
    SET(ADRFOO ${${NAME_ADDR_FOO}})
    GET_STD_CONFIG_NAME(${ADRFOO} CONFIG_FILE)
    ADD_SHELL_VAR(${CONFIG_FILE} SignalGUI_VERSION "${SIGNAL_GUI_VERSION}" "")
  ENDIF(PKCONFIG)
ENDMACRO(GEN_SIGNALGUI_VERSION NAME_ADDR_FOO)


# ----------------------------------------------------------------------
# It finalyzes the installation of the Polychrony environment.
# ----------------------------------------------------------------------
MACRO(FINALYZE_INSTALL)
  IF(PKCONFIG AND NOT DEFINED BIN_INSTALL)
    GEN_MESSAGE_INSTALL()    
  ENDIF(PKCONFIG AND NOT DEFINED BIN_INSTALL)
ENDMACRO(FINALYZE_INSTALL)
